@@ -3,6 +3,7 @@
 
 N.B. Implementation functions have a `_` prefix. Public functions do not. Don't worry to much about it. Just use the prefix-less versions when playing around, as they have all the debug niceties.
 """
+import re
 import collections
 import contextlib
 import logging
@@ -310,7 +311,7 @@ def _fas((n, noun)):
 Line 21:
 --------
 
-Now we enter the definition of Nock itself - ie, the `*` operato, _v=0r.
+Now we enter the definition of Nock itself - ie, the `*` operator.
 
     21 ::    *[a 0 b]          /[b a]
 
@@ -640,7 +641,8 @@ def _tar(noun):
     with _indent():
         if _wut(op) == YES:
             _d("<- 19 ::    *[a [b c] d]      [*[a b c] *[a d]]")
-            return (tar((subj, op)), tar((subj, obj)))
+            with _indent():
+                return (tar((subj, op)), tar((subj, obj)))
         else:
             if op == OP_FAS:
                 _d("<- 21 ::    *[a 0 b]          /[b a]")
@@ -654,7 +656,8 @@ def _tar(noun):
                 _d("<- 23 ::    *[a 2 b c]        *[*[a b] *[a c]]")
                 b = _fas((2, obj))
                 c = _fas((3, obj))
-                return tar((tar((subj, b)), tar((subj, c))))
+                with _indent():
+                    return tar((tar((subj, b)), tar((subj, c))))
 
             elif op == OP_WUT:
                 _d("<- 24 ::    *[a 3 b]          ?*[a b]")
@@ -674,36 +677,42 @@ def _tar(noun):
                 b = _fas((2, obj))
                 c = _fas((6, obj))
                 d = _fas((7, obj))
-                return tar((a, 2, (0, 1), 2, (1, c, d), (1, 0), 2, (1, 2, 3), (1, 0), 4, 4, b))
+                with _indent():
+                    return tar((a, 2, (0, 1), 2, (1, c, d), (1, 0), 2, (1, 2, 3), (1, 0), 4, 4, b))
 
             elif op == OP_H07:
                 _d("<- 29 ::    *[a 7 b c]        *[a 2 b 1 c]")
                 b = _fas((2, obj))
                 c = _fas((3, obj))
-                return tar((subj, 2, b, 1, c))
+                with _indent():
+                    return tar((subj, 2, b, 1, c))
 
             elif op == OP_H08:
                 _d("<- 30 ::    *[a 8 b c]        *[a 7 [[7 [0 1] b] 0 1] c]")
                 b = _fas((2, obj))
                 c = _fas((3, obj))
-                return tar((subj, 7, ((7, (0, 1), b), 0, 1), c))
+                with _indent():
+                    return tar((subj, 7, ((7, (0, 1), b), 0, 1), c))
 
             elif op == OP_H09:
                 _d("<- 31 ::    *[a 9 b c]        *[a 7 c 2 [0 1] 0 b]")
                 b = _fas((2, obj))
                 c = _fas((3, obj))
-                return tar((subj, 7, c, 2, (0, 1), 0, b))
+                with _indent():
+                    return tar((subj, 7, c, 2, (0, 1), 0, b))
 
             elif op == OP_H10:
                 hint = _fas((2, obj))
                 if _wut(hint) == YES:
                     _d("<- 32 ::    *[a 10 [b c] d]   *[a 8 c 7 [0 3] d]")
                     c = _fas((2, hint))
-                    return tar((subj, 8, c, 7, (0, 3), obj))
+                    with _indent():
+                        return tar((subj, 8, c, 7, (0, 3), obj))
                 else:
                     _d("<- 33 ::    *[a 10 b c]       *[a c]")
                     c = _fas((3, obj))
-                    return tar((subj, c))
+                    with _indent():
+                        return tar((subj, c))
 
 
 ### HELPERS, because WE NEED HELP.
@@ -738,7 +747,7 @@ def _indent():
 def _d(*args):
     """Log, at the given indentation level, the given logging arguments.
     """
-    level = DEBUG_LEVEL * '    '
+    level = DEBUG_LEVEL * ' '
     a = level + args[0]
     return logger.debug(a, *args[1:])
 
@@ -748,18 +757,40 @@ def _public(original_func, formatter):
     """
     def wrapper(noun):
         _d(formatter, _r(noun))
-        return original_func(noun)
+        result = original_func(noun)
+        with _indent():
+            _d(_r(result))
+        return result
     wrapper.__name__ = original_func.__name__.replace('_', '')
     wrapper.__doc__ = original_func.__doc__
     return wrapper
 
-# Public interface for Nock implementation functions.
+### Public interface for Nock implementation functions.
+#######################################################
 wut = _public(_wut, '?%s')
 lus = _public(_lus, '+%s')
 tis = _public(_tis, '=%s')
 fas = _public(_fas, '/%s')
 tar = _public(_tar, '*%s')
-nock = tar
+
+
+def nock(n):
+    """Reduce a Nock expression.
+
+    >>> nock((2, 0, 1))
+    2
+    >>> nock('[2 0 1]')
+    2
+    >>> nock('*[2 0 1]')
+    2
+    """
+    expr = n
+    if isinstance(n, basestring):
+        expr = parse(n)
+        if n.startswith('*'):
+            return expr
+
+    return tar(expr)
 
 
 def debug(on=True):
@@ -777,10 +808,83 @@ def debug(on=True):
         logger.setLevel(DEFAULT_LEVEL)
 
 
+### The PARSER
+##################
+TOKENS_CP = re.compile(r'\[|\]|[0-9]+|[*?=/+]')
+NUMBERS = set('0123456789')
+OPS = {
+    '/': fas,
+    '+': lus,
+    '*': tar,
+    '=': tis,
+    '?': wut,
+}
+
+
+def _construct(tk_iter, token):
+    """Construct and reduce Nock sub-expressions.
+    """
+    if token == '[':
+        out = []
+        token = tk_iter.next()
+        while token != ']':
+            out.append(_construct(tk_iter, token))
+            token = tk_iter.next()
+
+        return tuple(out)
+    if token in OPS:
+        return OPS[token](_construct(tk_iter, tk_iter.next()))
+    elif token in NUMBERS:
+        return int(token)
+
+    raise SyntaxError("Malformed Nock expression.")
+
+
+def parse(s):
+    """Nock parser.
+
+    Based on effbot's `iterator-based parser`_.
+
+    .. _iterator-based parser: http://effbot.org/zone/simple-iterator-parser.htm
+    """
+    tokens = iter(TOKENS_CP.findall(s))
+    return _construct(tokens, tokens.next())
+
+
+def main():
+    import sys
+    import readline
+    readline.parse_and_bind('tab: complete')
+    logging.basicConfig(level=logging.DEBUG, format='%(message)s')
+
+    print "Welcome to Nock! (`:q` or ^D to quit; `:debug on` to enter debug mode)"
+    print "    (If you're totally confused, read http://www.urbit.org/2013/08/22/Chapter-2-nock.html)"
+    print
+    try:
+        DEBUG = False
+        while True:
+            line = raw_input('-> ').strip()
+            if line == ':q':
+                break
+
+            elif line.startswith(':debug'):
+                if line.endswith('off'):
+                    DEBUG = False
+                elif line.endswith('on'):
+                    DEBUG = True
+                else:
+                    DEBUG = not DEBUG
+
+                debug()
+            else:
+                print _r(parse(line))
+                print
+    except EOFError:
+        pass
+
+    print "Good-bye!"
+    print
+    sys.exit()
+
 if __name__ == "__main__":
-    """
-    Since we can run doctests with nose, some day this should be an interactive
-    Nock prompt.
-    """
-    import doctest
-    doctest.testmod()
+    main()
